
#import <QuartzCore/QuartzCore.h>
#import "GameController.h"

@implementation GameController

@synthesize	gameView;

////////////////////////////////////////////////////////
//初期化
////////////////////////////////////////////////////////
- (id) initWithBaseView:(UIView*)baseview
{
	self = [super init];
	if (self != nil) {
		
		//ゲームビュー生成
		NSMutableArray*		tileArray = [NSMutableArray array];
		int					x, y, tileIndex = 0;
		for (y=0; y < TILE_H_COUNT; y++) {
			for (x=0; x < TILE_W_COUNT; x++) {
				CGRect	frame = CGRectMake(TILE_POS_X+(x*TILE_WIDTH), 
										   TILE_POS_Y+(y*TILE_HEIGHT), 
										   TILE_WIDTH, TILE_HEIGHT);
				GameBoardTile*	tile = [GameBoardTile gameBoardTileWith:frame 
																 State:0];
				tile.tileIndex = tileIndex;
				tileIndex++;
				[tileArray addObject:tile];
			}
		}
		self.gameView = [[[GameBoardView alloc] initWithFrame:CGRectMake(0, 0, 
																		 baseview.frame.size.width, 
																		 baseview.frame.size.height) 
															  Tiles:tileArray 
															BGImage:[UIImage imageNamed:@"bg00.png"]] autorelease];
		[baseview addSubview:self.gameView];
		self.gameView.delegate = self;
		
		//キャラクターを生成
		//うさぎ
		{
			NSMutableArray*	array = [NSMutableArray array];
			int			i;
			for(i=0; i < 15; i++){
				NSString*	name = [NSString stringWithFormat:@"rabit00_%03d.png", i];
				UIImage*	image = [UIImage imageNamed:name];
				[array addObject:image];
			}
			pieceView = [[GamePieceView alloc] initWithFrame:CGRectMake(0, 0, 48, 48) 
													  Images:array 
													Delegate:self];
			pieceView.type = TYPE_RABBIT;
			//配置
			[gameView addSubview:pieceView];
			GameBoardTile*	tile = [self.gameView tileAtIndex:44];
			tile.piece = pieceView;
			pieceView.tile = tile;
			pieceView.center = [tile center];
			//タイマースタート
			[pieceView timerStart];
		}
		
		//オオカミ
		{
			NSMutableArray*	array = [NSMutableArray array];
			int			i;
			for(i=0; i < 15; i++){
				NSString*	name = [NSString stringWithFormat:@"wlof00_%03d.png", i];
				UIImage*	image = [UIImage imageNamed:name];
				[array addObject:image];
			}
			enemyView = [[GamePieceView alloc] initWithFrame:CGRectMake(0, 0, 48, 48) 
													  Images:array 
													Delegate:self];
			enemyView.type = TYPE_WLOF;
			enemyView.speed = 50;
			//配置
			[gameView addSubview:enemyView];
			GameBoardTile*	tile = [self.gameView tileAtIndex:22];
			tile.piece = enemyView;
			enemyView.tile = tile;
			enemyView.center = [tile center];
			enemyView.state = STATE_SLEEP;
			//タイマースタート
			[enemyView timerStart];
		}
		
		
		
	}
	
	return	self;
}

////////////////////////////////////////////////////////
//解放
////////////////////////////////////////////////////////
- (void)dealloc {
	[pieceView release];
	[gameView release];
    [super dealloc];
}


////////////////////////////////////////////////////////
//近くにいるかチェック
////////////////////////////////////////////////////////
-(MovAngleType)nearPlayerPieceAngle:(GamePieceView*)piece
{	
	MovAngleType	ret = MOVANGL_NON;
	//オオカミの場合...
	if(piece.type==TYPE_WLOF) {
		int				tileIndex, targetIndex;
		//オオカミの現在の位置
		tileIndex = piece.tile.tileIndex;
		//うさぎの現在の位置
		targetIndex = pieceView.tile.tileIndex;
		if( tileIndex==targetIndex )
			ret = MOVANGL_FRONT;
		else if( tileIndex+1==targetIndex )
			ret = MOVANGL_RIGHT;
		else if( tileIndex-1==targetIndex )
			ret = MOVANGL_LEFT;
		else if( tileIndex+TILE_W_COUNT==targetIndex )
			ret = MOVANGL_FRONT;
		else if( tileIndex-TILE_W_COUNT==targetIndex )
			ret = MOVANGL_BACK;
		return	ret;
	}
	
	return	ret;
}


#pragma mark -
#pragma mark -- GameBoardViewDelegate --

////////////////////////////////////////////////////////
//タッチアップ
////////////////////////////////////////////////////////
-(void) gameBoardViewTouchUp:(GameBoardView*)gmView Location:(CGPoint)touchPt Taps:(int)taps Event:(UIEvent*)event
{
	GameBoardTile*	tile = [gmView tileAtPoint:touchPt];
	if( tile ){
		[pieceView moveWith:tile];
	}
}


#pragma mark -
#pragma mark -- GamePieceViewDelegate --

////////////////////////////////////////////////////////
//スタート
////////////////////////////////////////////////////////
-(void) gamePieceCAMoveStart:(GamePieceView*)piece Position:(CGPoint)pt
{
	//オオカミが攻撃開始
	if(piece.type==TYPE_WLOF && piece.state==STATE_ATTACK){
		//隣のマス目にいる
		if([self nearPlayerPieceAngle:piece]!=MOVANGL_NON){
			if( pieceView.state!=STATE_DAMAGE && pieceView.state!=STATE_DEAD ){
				pieceView.state = STATE_DAMAGE;
				if(pieceView.life <= 0){
					piece.state = STATE_STAY;
				}
			}
		}
	}
	//ウサギが移動開始
	if( piece.type==TYPE_RABBIT ){
		[enemyView moveWith:piece.tile];
	}
}

////////////////////////////////////////////////////////
//停止
////////////////////////////////////////////////////////
-(void) gamePieceCAMoveEnd:(GamePieceView*)piece Position:(CGPoint)pt Finish:(BOOL)isFinish
{
}

////////////////////////////////////////////////////////
//移動先のタイル
////////////////////////////////////////////////////////
-(GameBoardTile*)gamePieceNextMovTile:(GamePieceView*)piece
{
	int				x1, y1, x2, y2;
	int				tileIndex, targetIndex;
	GameBoardTile*	ret_tile=nil;
	
	//移動先
	targetIndex = piece.targetTile.tileIndex;
	x2 = (targetIndex % TILE_W_COUNT);
	y2 = (targetIndex / TILE_W_COUNT);
	//現在の位置
	tileIndex = piece.tile.tileIndex;
	x1 = (tileIndex % TILE_W_COUNT);
	y1 = (tileIndex / TILE_W_COUNT);
	
	if(y2 == y1 && x2 == x1 )
		return	nil;
	
	double	xdis = fabs( x2 - x1 );
	double	ydis = fabs( y2 - y1 );
	
	//右上
	if( y2 <= y1 && x2 >= x1 ){
		//縦の距離が長い
		if( xdis <= ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex-TILE_W_COUNT];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex+1];
		}
		//横の距離が長い
		else if( xdis > ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex+1];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex-TILE_W_COUNT];
		}
	}
	//右下
	else if( y2 >= y1 && x2 >= x1 ){
		//縦の距離が長い
		if( xdis <= ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex+TILE_W_COUNT];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex+1];
		}
		//横の距離が長い
		else if( xdis > ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex+1];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex+TILE_W_COUNT];
		}
	}
	//左下
	else if( y2 >= y1 && x2 <= x1 ){
		//縦の距離が長い
		if( xdis <= ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex+TILE_W_COUNT];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex-1];
		}
		//横の距離が長い
		else if( xdis > ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex-1];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex+TILE_W_COUNT];
		}
	}
	//左上
	else if( y2 <= y1 && x2 <= x1 ){
		//縦の距離が長い
		if( xdis <= ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex-TILE_W_COUNT];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex-1];
		}
		//横の距離が長い
		else if( xdis > ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex-1];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex-TILE_W_COUNT];
		}
	}
	
	if( ret_tile.piece )
		return	nil;
	
	return	ret_tile;
}

////////////////////////////////////////////////////////
//移動先のタイル
////////////////////////////////////////////////////////
-(GameBoardTile*)gamePieceTargetMovTile:(GamePieceView*)piece {
	GameBoardTile*	tile=nil;
	if([self nearPlayerPieceAngle:piece]==MOVANGL_NON){
		tile = [pieceView tile];
	}
	return	tile;
}

////////////////////////////////////////////////////////
//移動先を終了するか？
////////////////////////////////////////////////////////
-(BOOL)gamePieceIsMoveFinish:(GamePieceView*)piece {
	MovAngleType	ret = [self nearPlayerPieceAngle:piece];
	return	(ret!=MOVANGL_NON ? YES : NO);
}

////////////////////////////////////////////////////////
//攻撃するピースを返す
////////////////////////////////////////////////////////
-(GamePieceView*)gamePieceIsAttack:(GamePieceView*)piece
{
	if(piece.type==TYPE_WLOF){
		MovAngleType	angle = [piece angle:pieceView];
		if(pieceView.state!=STATE_DAMAGE && pieceView.state!=STATE_DEAD && angle!=MOVANGL_NON){
			return	pieceView;
		}
		else {
			return	nil;
		}
	}
	else {
		return	nil;
	}

}

////////////////////////////////////////////////////////
//死亡通知
////////////////////////////////////////////////////////
-(void)gamePieceDead:(GamePieceView*)piece
{
	//ウサギが死亡
	if( piece == pieceView ){
		//削除
		[pieceView removeFromSuperview];
		[pieceView timerStop];
		[pieceView release];
		pieceView = nil;
	}
}

@end
