
#import <UIKit/UIKit.h>

//アングル（配列にある画像のインデックス値も兼ねる）
typedef enum{
	MOVANGL_NON			=-1,	//
	
	MOVANGL_FRONT		= 0,	//正面
	MOVANGL_LEFT		= 3,	//左
	MOVANGL_RIGHT		= 6,	//右
	MOVANGL_BACK		= 9,	//後ろ
	
	MOVANGL_DEAD		=12,	//死亡
	MOVANGL_SLEEP		=13,	//眠り
	MOVANGL_DAMAGE		=14,	//ダメージ
	
} MovAngleType;

//タイプ
typedef enum{
	TYPE_RABBIT			=0,		//うさぎ
	TYPE_WLOF			=1,		//オオカミ
} PieceType;

//ステータス
typedef enum{
	STATE_STAY			=0,		//停止
	STATE_MOVEING		=1,		//移動
	
	STATE_DEAD			=2,		//死亡
	STATE_SLEEP			=3,		//眠り
	STATE_DAMAGE		=4,		//ダメージ
	
	STATE_ATTACKSTART	=5,		//攻撃開始
	STATE_ATTACK		=6,		//攻撃
} PieceState;



@class	GameBoardTile;

@interface GamePieceView : UIView {
	
	id				delegate;
	NSArray*		imageArray;
	int				imageIndex;
	int				animeIndex;
	CGPoint			moveLocation;
	float			speed;
	PieceType		type;
	PieceState		state;
	float			delayTimer;
	GameBoardTile*	tile;		//現在の位置
	GameBoardTile*	targetTile;	//最終移動位置
	GameBoardTile*	nextTile;	//次の移動位置
	int				life;
	double			delay_mov;
	NSTimer*		moveTimer;
	BOOL			isForceStop;
}

@property (nonatomic, assign) id				delegate;
@property (nonatomic, retain) NSArray*			imageArray;
@property (nonatomic, assign) int				imageIndex;
@property (nonatomic, assign) float				speed;
@property (nonatomic, assign) PieceType			type;
@property (nonatomic, assign) PieceState		state;
@property (nonatomic, retain) GameBoardTile*	tile;
@property (nonatomic, retain) GameBoardTile*	targetTile;
@property (nonatomic, retain) GameBoardTile*	nextTile;
@property (nonatomic, assign) int				life;
@property (nonatomic, assign) BOOL				isForceStop;

-(id)initWithFrame:(CGRect)frame Images:(NSArray*)images Delegate:(id)del;
-(void)moveWith:(GameBoardTile*)movTile;
-(void)movePosition:(CGPoint)position;
-(void)moveCancelPosition;

-(void)timerStart;
-(void)timerStop;
-(void)moveAction:(NSTimer*)timer;

-(void)update;

-(MovAngleType)angle:(GamePieceView*)piece2;
-(BOOL)attackCheck;

@end


//プロトコル宣言
@protocol GamePieceViewDelegate

-(void) gamePieceCAMoveStart:(GamePieceView*)piece Position:(CGPoint)pt;
-(void) gamePieceCAMoveEnd:(GamePieceView*)piece Position:(CGPoint)pt Finish:(BOOL)isFinish;

-(GameBoardTile*)gamePieceNextMovTile:(GamePieceView*)piece;
-(GameBoardTile*)gamePieceTargetMovTile:(GamePieceView*)piece;
-(BOOL)gamePieceIsMoveFinish:(GamePieceView*)piece;
-(GamePieceView*)gamePieceIsAttack:(GamePieceView*)piece;
-(void)gamePieceDead:(GamePieceView*)piece;

@end


